namespace dnnl {
namespace impl {
namespace gpu {
namespace intel {
namespace ocl {
const char *gen9_eltwise_kernel = R"==(/*******************************************************************************)==""\n"
R"==(* Copyright 2020-2024 Intel Corporation)==""\n"
R"==(*)==""\n"
R"==(* Licensed under the Apache License, Version 2.0 (the "License");)==""\n"
R"==(* you may not use this file except in compliance with the License.)==""\n"
R"==(* You may obtain a copy of the License at)==""\n"
R"==(*)==""\n"
R"==(* http:)==""\n"
R"==(*)==""\n"
R"==(* Unless required by applicable law or agreed to in writing, software)==""\n"
R"==(* distributed under the License is distributed on an "AS IS" BASIS,)==""\n"
R"==(* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.)==""\n"
R"==(* See the License for the specific language governing permissions and)==""\n"
R"==(* limitations under the License.)==""\n"
R"==(*******************************************************************************/)==""\n"
R"==(#include "gpu/intel/ocl/ocl_eltwise.h")==""\n"
R"==(#include "gpu/intel/ocl/ocl_post_ops.h")==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SIMD))) __kernel void gen9_eltwise_fwd()==""\n"
R"==(__global DATA_T *src, __global DATA_T *dst, dim_t nelems, float alpha,)==""\n"
R"==(float beta) {)==""\n"
R"==(const dim_t grsize = get_local_size(0);)==""\n"
R"==(const dim_t grid = get_group_id(0);)==""\n"
R"==(const dim_t sgid = get_sub_group_id();)==""\n"
R"==(const dim_t lid = get_sub_group_local_id();)==""\n"
R"==(const dim_t gid = get_global_id(0);)==""\n"
R"==(dim_t offset)==""\n"
R"==(= (grid * grsize + sgid * get_max_sub_group_size()) * VECT_DT_N;)==""\n"
R"==(__global BLOCK_DATA_T *read_pos = (__global BLOCK_DATA_T *)src + offset;)==""\n"
R"==(__global BLOCK_DATA_T *write_pos = (__global BLOCK_DATA_T *)dst + offset;)==""\n"
R"==(VECT_DATA_T val;)==""\n"
R"==(const int nel_per_read = SIMD * VECT_DT_N;)==""\n"
R"==(if (!NELEMS_OVERFLOW || offset + nel_per_read < nelems) {)==""\n"
R"==(val = AS_VECT_DATA_T(VECT_BLOCK_READ(read_pos));)==""\n"
R"==(} else {)==""\n"
R"==(dim_t pos = offset + lid;)==""\n"
R"==(for (int i = 0; i < VECT_DT_N && pos < nelems; ++i) {)==""\n"
R"==(val[i] = src[pos];)==""\n"
R"==(pos += SIMD;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(for (int i = 0; i < VECT_DT_N; ++i) {)==""\n"
R"==(val[i] = CONVERT_DATA_T()==""\n"
R"==(fwd_eltwise(DATA_TO_REF(val[i]), alpha, beta, 1.0f));)==""\n"
R"==(})==""\n"
R"==(if (!NELEMS_OVERFLOW || offset + nel_per_read < nelems) {)==""\n"
R"==(VECT_BLOCK_WRITE(write_pos, AS_VECT_BLOCK_DATA_T(val));)==""\n"
R"==(} else {)==""\n"
R"==(dim_t pos = offset + lid;)==""\n"
R"==(for (int i = 0; i < VECT_DT_N && pos < nelems; ++i) {)==""\n"
R"==(dst[pos] = val[i];)==""\n"
R"==(pos += SIMD;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SIMD))) __kernel void gen9_eltwise_bwd()==""\n"
R"==(__global DATA_T *src, __global DATA_T *diff_src,)==""\n"
R"==(__global DATA_T *diff_dst, dim_t nelems, float alpha, float beta) {)==""\n"
R"==(const dim_t grsize = get_local_size(0);)==""\n"
R"==(const dim_t grid = get_group_id(0);)==""\n"
R"==(const dim_t sgid = get_sub_group_id();)==""\n"
R"==(const dim_t lid = get_sub_group_local_id();)==""\n"
R"==(dim_t offset = (grid * grsize + sgid * SIMD) * VECT_DT_N;)==""\n"
R"==(__global BLOCK_DATA_T *src_pos = (__global BLOCK_DATA_T *)src + offset;)==""\n"
R"==(__global BLOCK_DATA_T *diff_pos)==""\n"
R"==(= (__global BLOCK_DATA_T *)diff_dst + offset;)==""\n"
R"==(__global BLOCK_DATA_T *write_pos)==""\n"
R"==(= (__global BLOCK_DATA_T *)diff_src + offset;)==""\n"
R"==(VECT_DATA_T val_dd;)==""\n"
R"==(VECT_DATA_T val_src;)==""\n"
R"==(const int nel_per_read = SIMD * VECT_DT_N;)==""\n"
R"==(if (!NELEMS_OVERFLOW || offset + nel_per_read < nelems) {)==""\n"
R"==(val_src = AS_VECT_DATA_T(VECT_BLOCK_READ(src_pos));)==""\n"
R"==(val_dd = AS_VECT_DATA_T(VECT_BLOCK_READ(diff_pos));)==""\n"
R"==(} else {)==""\n"
R"==(dim_t pos = offset + lid;)==""\n"
R"==(for (int i = 0; i < VECT_DT_N && pos < nelems; ++i) {)==""\n"
R"==(val_dd[i] = diff_dst[pos];)==""\n"
R"==(val_src[i] = src[pos];)==""\n"
R"==(pos += SIMD;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(for (int i = 0; i < VECT_DT_N; ++i) {)==""\n"
R"==(val_dd[i] = CONVERT_DATA_T(bwd_eltwise()==""\n"
R"==(DATA_TO_REF(val_dd[i]), DATA_TO_REF(val_src[i]), alpha, beta));)==""\n"
R"==(})==""\n"
R"==(if (!NELEMS_OVERFLOW || offset + nel_per_read < nelems) {)==""\n"
R"==(VECT_BLOCK_WRITE(write_pos, AS_VECT_BLOCK_DATA_T(val_dd));)==""\n"
R"==(} else {)==""\n"
R"==(dim_t pos = offset + lid;)==""\n"
R"==(for (int i = 0; i < VECT_DT_N && pos < nelems; ++i) {)==""\n"
R"==(diff_src[pos] = val_dd[i];)==""\n"
R"==(pos += SIMD;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==()==";
}
}
}
}
}