namespace dnnl {
namespace impl {
namespace gpu {
namespace intel {
namespace ocl {
const char *ocl_reduction_header = R"==(/*******************************************************************************)==""\n"
R"==(* Copyright 2024 Intel Corporation)==""\n"
R"==(*)==""\n"
R"==(* Licensed under the Apache License, Version 2.0 (the "License");)==""\n"
R"==(* you may not use this file except in compliance with the License.)==""\n"
R"==(* You may obtain a copy of the License at)==""\n"
R"==(*)==""\n"
R"==(* http:)==""\n"
R"==(*)==""\n"
R"==(* Unless required by applicable law or agreed to in writing, software)==""\n"
R"==(* distributed under the License is distributed on an "AS IS" BASIS,)==""\n"
R"==(* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.)==""\n"
R"==(* See the License for the specific language governing permissions and)==""\n"
R"==(* limitations under the License.)==""\n"
R"==(*******************************************************************************/)==""\n"
R"==(#ifndef GPU_INTEL_OCL_REDUCTION_OCL_REDUCTION_H)==""\n"
R"==(#define GPU_INTEL_OCL_REDUCTION_OCL_REDUCTION_H)==""\n"
R"==(#include "gpu/intel/ocl/ocl_utils.h")==""\n"
R"==(#define DEF_fp_minmax_abs(dt) \)==""\n"
R"==(dt __attribute__((overloadable)) min(dt lhs, dt rhs) { \)==""\n"
R"==(return fmin(lhs, rhs); \)==""\n"
R"==(} \)==""\n"
R"==(dt __attribute__((overloadable)) max(dt lhs, dt rhs) { \)==""\n"
R"==(return fmax(lhs, rhs); \)==""\n"
R"==(} \)==""\n"
R"==(dt __attribute__((overloadable)) abs(dt val) { return fabs(val); })==""\n"
R"==(DEF_fp_minmax_abs(float);)==""\n"
R"==(IF_HALF_SUPPORTED(DEF_fp_minmax_abs(half));)==""\n"
R"==(IF_DOUBLE_SUPPORTED(DEF_fp_minmax_abs(double));)==""\n"
R"==(#undef DEF_fp_minmax_abs)==""\n"
R"==(#define DEF_reduce(dt) \)==""\n"
R"==(dt __attribute__((overloadable)) \)==""\n"
R"==(reduce(int alg, dt lhs, dt rhs, float power) { \)==""\n"
R"==(switch (alg) { \)==""\n"
R"==(case (REDUCTION_MAX): return max(lhs, rhs); \)==""\n"
R"==(case (REDUCTION_MIN): return min(lhs, rhs); \)==""\n"
R"==(case (REDUCTION_MUL): return lhs * rhs; \)==""\n"
R"==(case (REDUCTION_LP_NORM_MAX): \)==""\n"
R"==(case (REDUCTION_LP_NORM_SUM): \)==""\n"
R"==(case (REDUCTION_LP_NORM_POWER_P_MAX): \)==""\n"
R"==(case (REDUCTION_LP_NORM_POWER_P_SUM): \)==""\n"
R"==(case (REDUCTION_LP_NORM_POWER_P): \)==""\n"
R"==(return (lhs + (dt)pow(convert_float(abs(rhs)), power)); \)==""\n"
R"==(case (REDUCTION_SUM): \)==""\n"
R"==(case (REDUCTION_FINAL_MAX): \)==""\n"
R"==(case (REDUCTION_FINAL_SUM): \)==""\n"
R"==(case (REDUCTION_PTH_ROOT_MAX): \)==""\n"
R"==(case (REDUCTION_PTH_ROOT_SUM): \)==""\n"
R"==(case (REDUCTION_MEAN): return lhs + rhs; \)==""\n"
R"==(default: return lhs; \)==""\n"
R"==(} \)==""\n"
R"==(})==""\n"
R"==(DEF_reduce(float);)==""\n"
R"==(DEF_reduce(int);)==""\n"
R"==(IF_DOUBLE_SUPPORTED(DEF_reduce(double));)==""\n"
R"==(IF_HALF_SUPPORTED(DEF_reduce(half));)==""\n"
R"==(#undef DEF_reduce)==""\n"
R"==(#if !DETERMINISTIC && ATOMICS_SUPPORTED)==""\n"
R"==(#define DEF_atomic_reduce(dt) \)==""\n"
R"==(dt __attribute__((overloadable)) atomic_reduce( \)==""\n"
R"==(int alg, __global ATOMIC(dt) * dst, dt rhs, float power) { \)==""\n"
R"==(switch (alg) { \)==""\n"
R"==(case (REDUCTION_MAX): return atomic_max_global(dst, rhs); \)==""\n"
R"==(case (REDUCTION_MIN): return atomic_min_global(dst, rhs); \)==""\n"
R"==(case (REDUCTION_LP_NORM_MAX): \)==""\n"
R"==(case (REDUCTION_LP_NORM_SUM): \)==""\n"
R"==(case (REDUCTION_LP_NORM_POWER_P_MAX): \)==""\n"
R"==(case (REDUCTION_LP_NORM_POWER_P_SUM): \)==""\n"
R"==(case (REDUCTION_LP_NORM_POWER_P): \)==""\n"
R"==(return atomic_add_global( \)==""\n"
R"==(dst, (dt)pow(convert_float(abs(rhs)), power)); \)==""\n"
R"==(case (REDUCTION_SUM): \)==""\n"
R"==(case (REDUCTION_FINAL_MAX): \)==""\n"
R"==(case (REDUCTION_FINAL_SUM): \)==""\n"
R"==(case (REDUCTION_PTH_ROOT_MAX): \)==""\n"
R"==(case (REDUCTION_PTH_ROOT_SUM): \)==""\n"
R"==(case (REDUCTION_MEAN): return atomic_add_global(dst, rhs); \)==""\n"
R"==(} \)==""\n"
R"==(printf("Atomic reduction on unsupported alg: %d\n", alg); \)==""\n"
R"==(return SPECIAL(dt, zero); \)==""\n"
R"==(})==""\n"
R"==(#if ATOMIC_FLOAT_SUPPORTED)==""\n"
R"==(DEF_atomic_reduce(float);)==""\n"
R"==(#endif)==""\n"
R"==(DEF_atomic_reduce(int);)==""\n"
R"==(#undef DEF_atomic_reduce)==""\n"
R"==(#endif)==""\n"
R"==(#define internal_def_init_acc(dt, min_val, max_val, zero_val, one_val) \)==""\n"
R"==(void __attribute__((overloadable)) init_acc(int alg, dt *val) { \)==""\n"
R"==(switch (alg) { \)==""\n"
R"==(case (REDUCTION_MAX): *val = min_val; return; \)==""\n"
R"==(case (REDUCTION_MIN): *val = max_val; return; \)==""\n"
R"==(case (REDUCTION_MUL): *val = one_val; return; \)==""\n"
R"==(default: *val = zero_val; return; \)==""\n"
R"==(} \)==""\n"
R"==(})==""\n"
R"==(#define CONCAt2(a, b) a##b)==""\n"
R"==(#define CONCAT2(a, b) CONCAt2(a, b))==""\n"
R"==(#define DEF_init_acc(dt, min_val, max_val, zero_val, one_val) \)==""\n"
R"==(internal_def_init_acc(dt, min_val, max_val, zero_val, one_val); \)==""\n"
R"==(internal_def_init_acc( \)==""\n"
R"==(CONCAT2(dt, 2), min_val, max_val, zero_val, one_val); \)==""\n"
R"==(internal_def_init_acc( \)==""\n"
R"==(CONCAT2(dt, 4), min_val, max_val, zero_val, one_val); \)==""\n"
R"==(internal_def_init_acc(CONCAT2(dt, 8), min_val, max_val, zero_val, one_val))==""\n"
R"==(DEF_init_acc(float, -FLT_MAX, FLT_MAX, 0.0f, 1.0f);)==""\n"
R"==(DEF_init_acc(int, INT_MIN, INT_MAX, 0, 1);)==""\n"
R"==(IF_DOUBLE_SUPPORTED(DEF_init_acc(double, -DBL_MAX, DBL_MAX, 0.0, 1.0));)==""\n"
R"==(IF_HALF_SUPPORTED(DEF_init_acc(half, -HALF_MAX, HALF_MAX, 0.0h, 1.0h));)==""\n"
R"==(#undef DEF_init_acc)==""\n"
R"==(#undef internal_def_init_acc)==""\n"
R"==(float finalize(int alg, float val, float div, float power, float eps) {)==""\n"
R"==(switch (alg) {)==""\n"
R"==(case (REDUCTION_MEAN): return val / div;)==""\n"
R"==(case (REDUCTION_LP_NORM_MAX):)==""\n"
R"==(case (REDUCTION_PTH_ROOT_MAX): return rootn(fmax(val, eps), power);)==""\n"
R"==(case (REDUCTION_LP_NORM_SUM):)==""\n"
R"==(case (REDUCTION_PTH_ROOT_SUM): return rootn(val + eps, power);)==""\n"
R"==(case (REDUCTION_LP_NORM_POWER_P_MAX):)==""\n"
R"==(case (REDUCTION_FINAL_MAX): return fmax(val, eps);)==""\n"
R"==(case (REDUCTION_LP_NORM_POWER_P_SUM):)==""\n"
R"==(case (REDUCTION_FINAL_SUM): return val + eps;)==""\n"
R"==(default: return val;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==()==";
}
}
}
}
}