namespace dnnl {
namespace impl {
namespace gpu {
namespace intel {
namespace ocl {
const char *ref_group_normalization_kernel = R"==(/*******************************************************************************)==""\n"
R"==(* Copyright 2024 Intel Corporation)==""\n"
R"==(*)==""\n"
R"==(* Licensed under the Apache License, Version 2.0 (the "License");)==""\n"
R"==(* you may not use this file except in compliance with the License.)==""\n"
R"==(* You may obtain a copy of the License at)==""\n"
R"==(*)==""\n"
R"==(* http:)==""\n"
R"==(*)==""\n"
R"==(* Unless required by applicable law or agreed to in writing, software)==""\n"
R"==(* distributed under the License is distributed on an "AS IS" BASIS,)==""\n"
R"==(* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.)==""\n"
R"==(* See the License for the specific language governing permissions and)==""\n"
R"==(* limitations under the License.)==""\n"
R"==(*******************************************************************************/)==""\n"
R"==(#include "gpu/intel/ocl/dispatch.h")==""\n"
R"==(#include "gpu/intel/ocl/ocl_post_ops.h")==""\n"
R"==(#include "gpu/intel/ocl/ocl_types.h")==""\n"
R"==(#define AUX_DATA_T float)==""\n"
R"==(#define GNORM_ACC POST_OP_DATA_T)==""\n"
R"==(#define GNORM_ACC_CONST_0 0.0f)==""\n"
R"==(#define GNORM_ACC_CONST_1 1.0f)==""\n"
R"==(#if IS_FWD)==""\n"
R"==(/**)==""\n"
R"==(* @ingroup OCL_KERNELS)==""\n"
R"==(* @brief Group Normalization reference implementation. Forward case.)==""\n"
R"==(*)==""\n"
R"==(* Reference implementation of the Group Normalization algorithm.)==""\n"
R"==(* This is forward case.)==""\n"
R"==(* Paralelized with "batch" and "group" dimentions)==""\n"
R"==(*)==""\n"
R"==(* @param [in] src Main input data array [batch, channel, depth, height, width])==""\n"
R"==(* @param [in,out] mean Mean array [batch, group])==""\n"
R"==(* @param [in,out] variance Variance array [batch, group])==""\n"
R"==(* @param [out] dst Main output data array [batch, channel, depth, height, width])==""\n"
R"==(* @param [in] scale Scale vector [channel])==""\n"
R"==(* @param [in] shift Shift vector [channel])==""\n"
R"==(* @param [in] src_scale Input data scale [1])==""\n"
R"==(* @param [in] dst_scale Output data scale [1])==""\n"
R"==(* @param [in] eps User defined constant)==""\n"
R"==(* )==""\n"
R"==(* POST_OP_ARGS Expands to ', const __global POST_OP_TYPE *po_0_binary_arg' where POST_OP_TYPE)==""\n"
R"==(* defined by post operation type API)==""\n"
R"==(*)==""\n"
R"==(*/)==""\n"
R"==(KERNEL_ATTR)==""\n"
R"==(__kernel void ref_gnorm_fwd(__global const SRC_DATA_T *src,)==""\n"
R"==(__global AUX_DATA_T *mean, __global AUX_DATA_T *variance,)==""\n"
R"==(__global DST_DATA_T *dst, __global const AUX_DATA_T *scale,)==""\n"
R"==(__global const AUX_DATA_T *shift, __global const AUX_DATA_T *src_scale,)==""\n"
R"==(__global const AUX_DATA_T *dst_scale,)==""\n"
R"==(const AUX_DATA_T eps POST_OP_ARGS) {)==""\n"
R"==(const size_t id_batch = GWS_GET_BATCH();)==""\n"
R"==(const size_t id_group = GWS_GET_NGROUPS();)==""\n"
R"==(const size_t C_PER_G = C / G;)==""\n"
R"==(const size_t channel_start = id_group * C_PER_G;)==""\n"
R"==(const size_t channel_end = channel_start + C_PER_G;)==""\n"
R"==(const GNORM_ACC stat_divisor = C_PER_G * D * H * W;)==""\n"
R"==(const size_t stat_off = id_batch * G + id_group;)==""\n"
R"==(GNORM_ACC mean_val = CALCULATE_STATS ? GNORM_ACC_CONST_0 : mean[stat_off];)==""\n"
R"==(if (CALCULATE_STATS) {)==""\n"
R"==(for_(size_t channel = channel_start; channel < channel_end; ++channel))==""\n"
R"==(for_(size_t depth = 0; depth < D; ++depth))==""\n"
R"==(for_(size_t height = 0; height < H; ++height))==""\n"
R"==(for (size_t width = 0; width < W; ++width) {)==""\n"
R"==(const size_t idx = SRC_OFF(id_batch, channel, depth, height, width);)==""\n"
R"==(const GNORM_ACC input_val = SRC_TO_REF(src[idx]);)==""\n"
R"==(mean_val += input_val;)==""\n"
R"==(})==""\n"
R"==(mean_val /= stat_divisor;)==""\n"
R"==(if (SAVE_STATS) mean[stat_off] = mean_val;)==""\n"
R"==(})==""\n"
R"==(GNORM_ACC variance_val)==""\n"
R"==(= CALCULATE_STATS ? GNORM_ACC_CONST_0 : variance[stat_off];)==""\n"
R"==(if (CALCULATE_STATS) {)==""\n"
R"==(for_(size_t channel = channel_start; channel < channel_end; ++channel))==""\n"
R"==(for_(size_t depth = 0; depth < D; ++depth))==""\n"
R"==(for_(size_t height = 0; height < H; ++height))==""\n"
R"==(for (size_t width = 0; width < W; ++width) {)==""\n"
R"==(const size_t idx = SRC_OFF(id_batch, channel, depth, height, width);)==""\n"
R"==(const GNORM_ACC input_val = SRC_TO_REF(src[idx]);)==""\n"
R"==(const GNORM_ACC var1 = input_val - mean_val;)==""\n"
R"==(variance_val += var1 * var1;)==""\n"
R"==(})==""\n"
R"==(variance_val /= stat_divisor; )==""\n"
R"==(if (SAVE_STATS) variance[stat_off] = variance_val;)==""\n"
R"==(})==""\n"
R"==(const GNORM_ACC src_scale_val)==""\n"
R"==(= WITH_SRC_SCALES ? src_scale[0] : GNORM_ACC_CONST_1;)==""\n"
R"==(const GNORM_ACC dst_scale_val)==""\n"
R"==(= WITH_DST_SCALES ? dst_scale[0] : GNORM_ACC_CONST_1;)==""\n"
R"==(const GNORM_ACC r_dst_scale_val = GNORM_ACC_CONST_1 / dst_scale_val;)==""\n"
R"==(const GNORM_ACC variance_rsqrt = rsqrt(variance_val + eps);)==""\n"
R"==(for (size_t channel = channel_start; channel < channel_end; ++channel) {)==""\n"
R"==(const GNORM_ACC scale_val = scale ? scale[channel] : GNORM_ACC_CONST_1;)==""\n"
R"==(const GNORM_ACC shift_val = shift ? shift[channel] : GNORM_ACC_CONST_0;)==""\n"
R"==(for_(size_t depth = 0; depth < D; ++depth))==""\n"
R"==(for_(size_t height = 0; height < H; ++height))==""\n"
R"==(for (size_t width = 0; width < W; ++width) {)==""\n"
R"==(const size_t idx = SRC_OFF(id_batch, channel, depth, height, width);)==""\n"
R"==(const GNORM_ACC input_val = SRC_TO_REF(src[idx]);)==""\n"
R"==(GNORM_ACC result_val = (input_val - mean_val) * variance_rsqrt;)==""\n"
R"==(result_val = scale_val * result_val + shift_val;)==""\n"
R"==(result_val *= src_scale_val;)==""\n"
R"==(GNORM_ACC post_op_acc)==""\n"
R"==(= WITH_SUM ? DST_TO_REF(dst[idx]) : GNORM_ACC_CONST_0;)==""\n"
R"==(#if NDIMS == 3)==""\n"
R"==(APPLY_POST_OPS_SERIAL(result_val, GNORM_ACC, post_op_acc, GNORM_ACC,)==""\n"
R"==(id_batch, 1, channel, 1, width, 1, 0, 1, 0, 1, 0, 1);)==""\n"
R"==(#elif NDIMS == 4)==""\n"
R"==(APPLY_POST_OPS_SERIAL(result_val, GNORM_ACC, post_op_acc, GNORM_ACC,)==""\n"
R"==(id_batch, 1, channel, 1, height, 1, width, 1, 0, 1, 0, 1);)==""\n"
R"==(#elif NDIMS == 5)==""\n"
R"==(APPLY_POST_OPS_SERIAL(result_val, GNORM_ACC, post_op_acc, GNORM_ACC,)==""\n"
R"==(id_batch, 1, channel, 1, depth, 1, height, 1, width, 1, 0,)==""\n"
R"==(1);)==""\n"
R"==(#else)==""\n"
R"==(APPLY_POST_OPS_SERIAL(result_val, GNORM_ACC, post_op_acc, GNORM_ACC,)==""\n"
R"==(id_batch, 1, channel, 1, 0, 1, 0, 1, 0, 1, 0, 1);)==""\n"
R"==(#endif)==""\n"
R"==(result_val *= r_dst_scale_val;)==""\n"
R"==(dst[idx] = TO_DST(result_val);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(#else )==""\n"
R"==(/**)==""\n"
R"==(* @ingroup OCL_KERNELS)==""\n"
R"==(* @brief Group Normalization reference implementation. Backward case.)==""\n"
R"==(*)==""\n"
R"==(* Reference implementation of the Group Normalization algorithm.)==""\n"
R"==(* This is backward case.)==""\n"
R"==(* Paralelized with "channel" dimention)==""\n"
R"==(*)==""\n"
R"==(* @param [in] src Main input data array [batch, channel, depth, height, width])==""\n"
R"==(* @param [in,out] mean Mean array [batch, group])==""\n"
R"==(* @param [in,out] variance Variance array [batch, group])==""\n"
R"==(* @param [in] diff_dst Main input weights array [batch, channel, depth, height, width])==""\n"
R"==(* @param [in] scale Scale vector [channel])==""\n"
R"==(* @param [out] out_diff_src Main output data array [batch, channel, depth, height, width])==""\n"
R"==(* @param [out] out_diff_scale Calculated scale vactor [channel])==""\n"
R"==(* @param [out] out_diff_shift Calculated shift vactor [channel])==""\n"
R"==(* @param [in] eps User defined constant)==""\n"
R"==(*)==""\n"
R"==(*/)==""\n"
R"==(KERNEL_ATTR)==""\n"
R"==(__kernel void ref_gnorm_bwd(__global const SRC_DATA_T *src,)==""\n"
R"==(__global AUX_DATA_T *mean, __global AUX_DATA_T *variance,)==""\n"
R"==(__global const DST_DATA_T *diff_dst, __global const AUX_DATA_T *scale,)==""\n"
R"==(__global SRC_DATA_T *out_diff_src, __global AUX_DATA_T *out_diff_scale,)==""\n"
R"==(__global AUX_DATA_T *out_diff_shift, const AUX_DATA_T eps) {)==""\n"
R"==(const size_t id_channel = GWS_GET_CHANNEL();)==""\n"
R"==(const size_t C_PER_G = C / G;)==""\n"
R"==(const GNORM_ACC CSP = C_PER_G * D * H * W;)==""\n"
R"==(const GNORM_ACC CSP_recip = GNORM_ACC_CONST_1 / CSP;)==""\n"
R"==(const size_t id_group = id_channel / C_PER_G;)==""\n"
R"==(GNORM_ACC diff_scale = GNORM_ACC_CONST_0;)==""\n"
R"==(GNORM_ACC diff_shift = GNORM_ACC_CONST_0;)==""\n"
R"==(for (size_t batch = 0; batch < MB; ++batch) {)==""\n"
R"==(const size_t idx_stat = batch * G + id_group;)==""\n"
R"==(const GNORM_ACC mean_val = mean[idx_stat];)==""\n"
R"==(const GNORM_ACC variance_val = variance[idx_stat];)==""\n"
R"==(const GNORM_ACC variance_recip = rsqrt(variance_val + eps);)==""\n"
R"==(for_(size_t depth = 0; depth < D; ++depth))==""\n"
R"==(for_(size_t height = 0; height < H; ++height))==""\n"
R"==(for (size_t width = 0; width < W; ++width) {)==""\n"
R"==(const size_t idx = SRC_OFF(batch, id_channel, depth, height, width);)==""\n"
R"==(const GNORM_ACC src_val = SRC_TO_REF(src[idx]);)==""\n"
R"==(const GNORM_ACC diff_dst_val = DST_TO_REF(diff_dst[idx]);)==""\n"
R"==(diff_scale += (src_val - mean_val) * diff_dst_val * variance_recip;)==""\n"
R"==(diff_shift += diff_dst_val;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(if (out_diff_scale) out_diff_scale[id_channel] = diff_scale;)==""\n"
R"==(if (out_diff_shift) out_diff_shift[id_channel] = diff_shift;)==""\n"
R"==(GNORM_ACC scale_val = scale ? scale[id_channel] : GNORM_ACC_CONST_1;)==""\n"
R"==(for (size_t batch = 0; batch < MB; ++batch) {)==""\n"
R"==(const size_t idx_stat = batch * G + id_group;)==""\n"
R"==(const GNORM_ACC variance_val = variance[idx_stat];)==""\n"
R"==(const GNORM_ACC mean_val = mean[idx_stat];)==""\n"
R"==(const GNORM_ACC variance_recip = rsqrt(variance_val + eps);)==""\n"
R"==(for_(size_t depth = 0; depth < D; ++depth))==""\n"
R"==(for_(size_t height = 0; height < H; ++height))==""\n"
R"==(for (size_t width = 0; width < W; ++width) {)==""\n"
R"==(const size_t idx = SRC_OFF(batch, id_channel, depth, height, width);)==""\n"
R"==(const GNORM_ACC src_val = SRC_TO_REF(src[idx]);)==""\n"
R"==(GNORM_ACC result = DST_TO_REF(diff_dst[idx]);)==""\n"
R"==(if (CALCULATE_STATS) {)==""\n"
R"==(const GNORM_ACC diff_stat = diff_shift * CSP_recip)==""\n"
R"==(+ (src_val - mean_val) * diff_scale * variance_recip)==""\n"
R"==(* CSP_recip;)==""\n"
R"==(result -= diff_stat;)==""\n"
R"==(})==""\n"
R"==(result *= scale_val * variance_recip;)==""\n"
R"==(out_diff_src[idx] = TO_SRC(result);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(#endif )==""\n"
R"==()==";
}
}
}
}
}