namespace dnnl {
namespace impl {
namespace gpu {
namespace intel {
namespace ocl {
const char *reusable_vectorized_lnorm_kernel = R"==(/*******************************************************************************)==""\n"
R"==(* Copyright 2023-2024 Intel Corporation)==""\n"
R"==(*)==""\n"
R"==(* Licensed under the Apache License, Version 2.0 (the "License");)==""\n"
R"==(* you may not use this file except in compliance with the License.)==""\n"
R"==(* You may obtain a copy of the License at)==""\n"
R"==(*)==""\n"
R"==(* http:)==""\n"
R"==(*)==""\n"
R"==(* Unless required by applicable law or agreed to in writing, software)==""\n"
R"==(* distributed under the License is distributed on an "AS IS" BASIS,)==""\n"
R"==(* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.)==""\n"
R"==(* See the License for the specific language governing permissions and)==""\n"
R"==(* limitations under the License.)==""\n"
R"==(*******************************************************************************/)==""\n"
R"==(#include "gpu/intel/ocl/dispatch.h")==""\n"
R"==(#include "gpu/intel/ocl/ocl_math_utils.h")==""\n"
R"==(#include "gpu/intel/ocl/ocl_types.h")==""\n"
R"==(#include "gpu/intel/ocl/types_interop.h")==""\n"
R"==(#define VEC_SUM_DEFINE(TYPE) \)==""\n"
R"==(__attribute__((overloadable)) float vec_sum(TYPE val) { \)==""\n"
R"==(return vec_sum(val.even + val.odd); \)==""\n"
R"==(})==""\n"
R"==(__attribute__((overloadable)) float vec_sum(float val) {)==""\n"
R"==(return val;)==""\n"
R"==(})==""\n"
R"==(VEC_SUM_DEFINE(float2))==""\n"
R"==(VEC_SUM_DEFINE(float4))==""\n"
R"==(VEC_SUM_DEFINE(float8))==""\n"
R"==(__attribute__((intel_reqd_sub_group_size(SG_SIZE))) __kernel void)==""\n"
R"==(lnorm_reusable_vectorized(__global SRC_DATA_T *src, __global float *mean,)==""\n"
R"==(__global float *variance, dim_t reduce_size, __global DST_DATA_T *dst,)==""\n"
R"==(__global WEI_DATA_T *scale, __global WEI_DATA_T *shift, float eps,)==""\n"
R"==(__global float *src_scale, __global float *dst_scale, int greads,)==""\n"
R"==(float rrs, dispatch_gws_rt_params_t gws_params) {)==""\n"
R"==(src = (GWS_GET_BUFFER_POS(SRC, gws_params, src)) - get_sub_group_local_id();)==""\n"
R"==(FLT_ACC_DATA_T local_variance = 0.f;)==""\n"
R"==(FLT_ACC_DATA_T local_mean = 0.f;)==""\n"
R"==(if (CALCULATE_STATS) {)==""\n"
R"==(FLT_ACC_DATA_T sum = 0;)==""\n"
R"==(unroll_for_by(N_UNROLL)(int sg_idx = 0; sg_idx < reduce_size;)==""\n"
R"==(sg_idx += SG_STRIDE) {)==""\n"
R"==(VECT_FLOAT_T val)==""\n"
R"==(= CONVERT_VECT_FLOAT_T(AS_VECT_DATA_T(VECT_BLOCK_READ()==""\n"
R"==((const __global BLOCK_DATA_T *)(&src[sg_idx]))));)==""\n"
R"==(sum += vec_sum(val);)==""\n"
R"==(})==""\n"
R"==(local_mean = sub_group_reduce_add(sum) * rrs;)==""\n"
R"==(FLT_ACC_DATA_T sumsq = 0;)==""\n"
R"==(unroll_for_by(N_UNROLL)(int i = 0; i < greads; i++) {)==""\n"
R"==(VECT_FLOAT_T val;)==""\n"
R"==(val = CONVERT_VECT_FLOAT_T(AS_VECT_DATA_T(VECT_BLOCK_READ(()==""\n"
R"==(const __global BLOCK_DATA_T *)(&src[i * SG_STRIDE])))))==""\n"
R"==(- local_mean;)==""\n"
R"==(val *= val;)==""\n"
R"==(sumsq += vec_sum(val);)==""\n"
R"==(})==""\n"
R"==(local_variance = sub_group_reduce_add(sumsq) * rrs;)==""\n"
R"==(} else {)==""\n"
R"==(mean = GWS_GET_BUFFER_POS(STAT, gws_params, mean);)==""\n"
R"==(variance = GWS_GET_BUFFER_POS(STAT, gws_params, variance);)==""\n"
R"==(local_mean = *mean;)==""\n"
R"==(local_variance = *variance;)==""\n"
R"==(})==""\n"
R"==(if (USE_SCALE))==""\n"
R"==(scale = GWS_GET_BUFFER_POS(SS, gws_params, scale))==""\n"
R"==(+ ((greads - 1) * SG_STRIDE);)==""\n"
R"==(if (USE_SHIFT))==""\n"
R"==(shift = GWS_GET_BUFFER_POS(SS, gws_params, shift))==""\n"
R"==(+ ((greads - 1) * SG_STRIDE);)==""\n"
R"==(FLT_ACC_DATA_T sqrt_variance = rsqrt(local_variance + eps);)==""\n"
R"==(__global DST_DATA_T *dst_vect = (GWS_GET_BUFFER_POS(DST, gws_params, dst)))==""\n"
R"==(- get_sub_group_local_id() + ((greads - 1) * SG_STRIDE);)==""\n"
R"==(float src_scale_val = src_scale ? *src_scale : 1.f;)==""\n"
R"==(float dst_scale_val = dst_scale ? native_recip(*dst_scale) : 1.f;)==""\n"
R"==(unroll_for_by(N_UNROLL)(int i = greads - 1; i >= 0; i--) {)==""\n"
R"==(VECT_FLOAT_T res)==""\n"
R"==(= CONVERT_VECT_FLOAT_T(AS_VECT_DATA_T(VECT_BLOCK_READ(()==""\n"
R"==(const __global BLOCK_DATA_T *)(&src[i * SG_STRIDE])))))==""\n"
R"==(- local_mean;)==""\n"
R"==(res *= sqrt_variance;)==""\n"
R"==(if (USE_SCALE) res *= LOAD_VECT_WEI(scale);)==""\n"
R"==(if (USE_SHIFT) res += LOAD_VECT_WEI(shift);)==""\n"
R"==(res *= src_scale_val;)==""\n"
R"==(res *= dst_scale_val;)==""\n"
R"==(VECT_DST_BLOCK_WRITE(dst_vect, CONVERT_VECTOR_DST_DATA_T(res));)==""\n"
R"==(dst_vect -= SG_STRIDE;)==""\n"
R"==(if (USE_SCALE) scale -= SG_STRIDE;)==""\n"
R"==(if (USE_SHIFT) shift -= SG_STRIDE;)==""\n"
R"==(})==""\n"
R"==(if (SAVE_STATS && get_sub_group_local_id() == 0) {)==""\n"
R"==(mean = GWS_GET_BUFFER_POS(STAT, gws_params, mean);)==""\n"
R"==(variance = GWS_GET_BUFFER_POS(STAT, gws_params, variance);)==""\n"
R"==(*mean = local_mean;)==""\n"
R"==(*variance = local_variance;)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==()==";
}
}
}
}
}