namespace dnnl {
namespace impl {
namespace gpu {
namespace intel {
namespace ocl {
const char *rnn_common_header = R"==(/*******************************************************************************)==""\n"
R"==(* Copyright 2019-2024 Intel Corporation)==""\n"
R"==(*)==""\n"
R"==(* Licensed under the Apache License, Version 2.0 (the "License");)==""\n"
R"==(* you may not use this file except in compliance with the License.)==""\n"
R"==(* You may obtain a copy of the License at)==""\n"
R"==(*)==""\n"
R"==(* http:)==""\n"
R"==(*)==""\n"
R"==(* Unless required by applicable law or agreed to in writing, software)==""\n"
R"==(* distributed under the License is distributed on an "AS IS" BASIS,)==""\n"
R"==(* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.)==""\n"
R"==(* See the License for the specific language governing permissions and)==""\n"
R"==(* limitations under the License.)==""\n"
R"==(*******************************************************************************/)==""\n"
R"==(#ifndef GPU_INTEL_OCL_RNN_RNN_COMMON_H)==""\n"
R"==(#define GPU_INTEL_OCL_RNN_RNN_COMMON_H)==""\n"
R"==(#include "gpu/intel/ocl/ocl_types.h")==""\n"
R"==(#include "gpu/intel/ocl/types_interop.h")==""\n"
R"==(#if OUTPUT_DT_U8)==""\n"
R"==(#define TO_OUTPUT(x) convert_uchar_sat_rte(x))==""\n"
R"==(#elif OUTPUT_DT_S8)==""\n"
R"==(#define TO_OUTPUT(x) convert_char_sat_rte(x))==""\n"
R"==(#elif OUTPUT_DT_S32)==""\n"
R"==(#define TO_OUTPUT(x) convert_int_sat_rte(x))==""\n"
R"==(#else)==""\n"
R"==(#define TO_OUTPUT(x) (x))==""\n"
R"==(#endif)==""\n"
R"==(#if INPUT_DT_BF16)==""\n"
R"==(#define TO_REF(x) cvt_bf16_to_f32(x))==""\n"
R"==(#else)==""\n"
R"==(#define TO_REF(x) convert_float(x))==""\n"
R"==(#endif)==""\n"
R"==(#if AUX_DT_F16)==""\n"
R"==(#define TO_AUX(x) convert_half(x))==""\n"
R"==(#elif AUX_DT_F32)==""\n"
R"==(#define TO_AUX(x) convert_float(x))==""\n"
R"==(#else)==""\n"
R"==(#error "Unimplemented AUX_DATA_T type")==""\n"
R"==(#endif)==""\n"
R"==(#if NEED_BIAS_ATOMIC_REDUCE)==""\n"
R"==(#define MAYBE_ATOMIC volatile __global)==""\n"
R"==(#define DIFF_BIAS_DATA_T CONCAT2(atomic_, DIFF_DATA_T))==""\n"
R"==(#else)==""\n"
R"==(#define MAYBE_ATOMIC __global)==""\n"
R"==(#define DIFF_BIAS_DATA_T DIFF_DATA_T)==""\n"
R"==(#endif)==""\n"
R"==(#define OFFTYPE ulong)==""\n"
R"==(#define TO_WS_STATE(x) TO_SRC(x))==""\n"
R"==(#define OFF6(i0, D0, i1, D1, i2, D2, i3, D3, i4, D4, i5, D5) \)==""\n"
R"==(((((((i0) * (D1) + (i1)) * (D2) + (i2)) * (D3) + (i3)) * (D4) + (i4)) \)==""\n"
R"==(* (D5) \)==""\n"
R"==(+ (i5)))==""\n"
R"==(#define OFF5(i0, D0, i1, D1, i2, D2, i3, D3, i4, D4) \)==""\n"
R"==((((((i0) * (D1) + (i1)) * (D2) + (i2)) * (D3) + (i3)) * (D4) + (i4)))==""\n"
R"==(#define OFF4(i0, D0, i1, D1, i2, D2, i3, D3) \)==""\n"
R"==(((((i0) * (D1) + (i1)) * (D2) + (i2)) * (D3) + (i3)))==""\n"
R"==(#define OFF3(i0, D0, i1, D1, i2, D2) (((i0) * (D1) + (i1)) * (D2) + (i2)))==""\n"
R"==(#define OFF2(i0, D0, i1, D1) ((i0) * (D1) + (i1)))==""\n"
R"==(const int vanilla_rnn_n_gates = 1;)==""\n"
R"==(const int vanilla_rnn_n_bias = 1;)==""\n"
R"==(const int vanilla_lstm_n_gates = 4;)==""\n"
R"==(const int vanilla_lstm_n_bias = 4;)==""\n"
R"==(#if CELL_KIND == VANILLA_RNN)==""\n"
R"==(const int n_gates = vanilla_rnn_n_gates;)==""\n"
R"==(const int n_bias = vanilla_rnn_n_bias;)==""\n"
R"==(#elif CELL_KIND == VANILLA_LSTM)==""\n"
R"==(const int n_gates = vanilla_lstm_n_gates;)==""\n"
R"==(const int n_bias = vanilla_lstm_n_bias;)==""\n"
R"==(#elif CELL_KIND == LBR_GRU)==""\n"
R"==(const int n_gates = 3;)==""\n"
R"==(const int n_bias = 4;)==""\n"
R"==(#elif CELL_KIND == VANILLA_GRU)==""\n"
R"==(const int n_gates = 3;)==""\n"
R"==(const int n_bias = 3;)==""\n"
R"==(#else)==""\n"
R"==(#error "Unimplemented cell kind")==""\n"
R"==(#endif)==""\n"
R"==(int comp_off(int n_dir, int dhc, int i0, int i1, int i2, int i3) {)==""\n"
R"==(return (((i0 * n_dir + i1) * n_bias + i2) * dhc + i3);)==""\n"
R"==(})==""\n"
R"==(int off_ws_state(int n_layer, int n_dir, int n_iter, int batch,)==""\n"
R"==(int states_ws_ld, int i0_, int i1, int i2_, int i3, int i4) {)==""\n"
R"==(int i0 = COPY_SRC_LAYER ? i0_ + 1 : i0_;)==""\n"
R"==(int i0_size = COPY_SRC_LAYER ? n_layer + 1 : n_layer;)==""\n"
R"==(int i2 = i2_ + 1;)==""\n"
R"==(return OFF5(i0, i0_size, i1, n_dir, i2, n_iter + 1, i3, batch, i4,)==""\n"
R"==(states_ws_ld);)==""\n"
R"==(})==""\n"
R"==(int off_ws_c_state(int n_layer, int n_dir, int n_iter, int batch,)==""\n"
R"==(int states_ws_ld, int i0_, int i1, int i2_, int i3, int i4) {)==""\n"
R"==(int i0 = i0_;)==""\n"
R"==(int i0_size = n_layer;)==""\n"
R"==(int i2 = i2_ + 1;)==""\n"
R"==(return OFF5(i0, i0_size, i1, n_dir, i2, n_iter + 1, i3, batch, i4,)==""\n"
R"==(states_ws_ld);)==""\n"
R"==(})==""\n"
R"==(int off_ws_gates(int n_dir, int n_iter, int batch, int gates_ws_ld, int dhc,)==""\n"
R"==(int i0, int i1, int i2, int i3, int i4, int i5) {)==""\n"
R"==(return i0 * n_dir * n_iter * batch * gates_ws_ld)==""\n"
R"==(+ i1 * n_iter * batch * gates_ws_ld + i2 * batch * gates_ws_ld)==""\n"
R"==(+ i3 * gates_ws_ld + i4 * dhc + i5;)==""\n"
R"==(})==""\n"
R"==(int off_ws_bias()==""\n"
R"==(int n_layer, int n_dir, int dhc, int i0, int i1, int i2, int i3) {)==""\n"
R"==(return OFF4(i0, n_layer, i1, n_dir, i2, n_bias, i3, dhc);)==""\n"
R"==(})==""\n"
R"==(int off_ws_grid_offset(int n_layer, int n_dir, int n_iter, int batch, int dhc,)==""\n"
R"==(int i0, int i1, int i2, int i3, int i4) {)==""\n"
R"==(return OFF5(i0, n_layer, i1, n_dir, i2, n_iter, i3, batch, i4, dhc);)==""\n"
R"==(})==""\n"
R"==(int off_ker_bias(int dhc, int i0, int i1) {)==""\n"
R"==(return OFF2(i0, n_gates, i1, dhc);)==""\n"
R"==(})==""\n"
R"==(int off_scratch_diff_states(int n_layer, int n_dir, int n_states, int n_iter,)==""\n"
R"==(int batch, int scratch_diff_states_ld, int i0, int i1, int i2, int i3,)==""\n"
R"==(int i4, int i5) {)==""\n"
R"==(bool have_result_layer = COPY_DIFF_SRC_LAYER || n_layer > 1;)==""\n"
R"==(int i0_size = n_layer - 1 + COPY_DIFF_DST_LAYER + have_result_layer;)==""\n"
R"==(if (!have_result_layer) { i0--; })==""\n"
R"==(int i2_size = COPY_DIFF_DST_LAYER || COPY_DIFF_SRC_LAYER || n_layer != 1)==""\n"
R"==(? n_states + 1)==""\n"
R"==(: n_states;)==""\n"
R"==(int i3_size = n_iter + 1;)==""\n"
R"==(if (i0_size <= 1) {)==""\n"
R"==(if (i0_size <= 0) {)==""\n"
R"==(i3_size = 2;)==""\n"
R"==(i3 %= i3_size;)==""\n"
R"==(})==""\n"
R"==(return OFF5(i1, conf_.n_dir, i2, i2_size, i3, i3_size, i4, batch, i5,)==""\n"
R"==(scratch_diff_states_ld);)==""\n"
R"==(})==""\n"
R"==(return OFF6(i0, i0_size, i1, n_dir, i2, i2_size, i3, i3_size, i4, batch, i5,)==""\n"
R"==(scratch_diff_states_ld);)==""\n"
R"==(})==""\n"
R"==(int off_scratch_dhg1(int batch, int scratch_diff_states_ld, int i0, int i1) {)==""\n"
R"==(return OFF2(i0, batch, i1, scratch_diff_states_ld);)==""\n"
R"==(})==""\n"
R"==(int off_scratch_cell(int batch, int states_ws_ld, int i0, int i1) {)==""\n"
R"==(return OFF2(i0, batch, i1, states_ws_ld);)==""\n"
R"==(})==""\n"
R"==(int cell_ws_state(int states_ws_ld, int i, int j) {)==""\n"
R"==(return i * states_ws_ld + j;)==""\n"
R"==(})==""\n"
R"==(int cell_ws_gates(int gates_ws_ld, int dhc, int i, int n, int j) {)==""\n"
R"==(return i * gates_ws_ld + n * dhc + j;)==""\n"
R"==(})==""\n"
R"==(int cell_ws_grid_comp(int dhc, int i3, int i4) {)==""\n"
R"==(return i3 * dhc + i4;)==""\n"
R"==(})==""\n"
R"==(int cell_scratch_mem(int scratch_gates_ld, int dhc, int i, int n, int j) {)==""\n"
R"==(return i * scratch_gates_ld + n * dhc + j;)==""\n"
R"==(})==""\n"
R"==(int cell_scratch_diff_states()==""\n"
R"==(int batch, int scratch_diff_states_ld, int i4, int i5) {)==""\n"
R"==(return (i4 * scratch_diff_states_ld + i5);)==""\n"
R"==(})==""\n"
R"==(int bias_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % BIAS_B0) * BIAS_SB0 + (x0 / BIAS_B0) * (int)s[0])==""\n"
R"==(+ (x1 % BIAS_B1) * BIAS_SB1 + (x1 / BIAS_B1) * (int)s[1])==""\n"
R"==(+ (x2 % BIAS_B2) * BIAS_SB2 + (x2 / BIAS_B2) * (int)s[2])==""\n"
R"==(+ (x3 % BIAS_B3) * BIAS_SB3 + (x3 / BIAS_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(int src_l_off(int64x3_t strides, int iter, int batch, int slc) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((iter % SRC_L_B0) * SRC_L_SB0 + (iter / SRC_L_B0) * (int)s[0])==""\n"
R"==(+ (batch % SRC_L_B1) * SRC_L_SB1 + (batch / SRC_L_B1) * (int)s[1])==""\n"
R"==(+ (slc % SRC_L_B2) * SRC_L_SB2 + (slc / SRC_L_B2) * (int)s[2]);)==""\n"
R"==(})==""\n"
R"==(int src_i_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % SRC_I_B0) * SRC_I_SB0 + (x0 / SRC_I_B0) * (int)s[0])==""\n"
R"==(+ (x1 % SRC_I_B1) * SRC_I_SB1 + (x1 / SRC_I_B1) * (int)s[1])==""\n"
R"==(+ (x2 % SRC_I_B2) * SRC_I_SB2 + (x2 / SRC_I_B2) * (int)s[2])==""\n"
R"==(+ (x3 % SRC_I_B3) * SRC_I_SB3 + (x3 / SRC_I_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(#if WITH_SRC_ITER_C)==""\n"
R"==(int src_i_c_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % SRC_I_C_B0) * SRC_I_C_SB0 + (x0 / SRC_I_C_B0) * (int)s[0])==""\n"
R"==(+ (x1 % SRC_I_C_B1) * SRC_I_C_SB1 + (x1 / SRC_I_C_B1) * (int)s[1])==""\n"
R"==(+ (x2 % SRC_I_C_B2) * SRC_I_C_SB2 + (x2 / SRC_I_C_B2) * (int)s[2])==""\n"
R"==(+ (x3 % SRC_I_C_B3) * SRC_I_C_SB3 + (x3 / SRC_I_C_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(int dst_l_off(int64x3_t strides, int x0, int x1, int x2) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % DST_L_B0) * DST_L_SB0 + (x0 / DST_L_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DST_L_B1) * DST_L_SB1 + (x1 / DST_L_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DST_L_B2) * DST_L_SB2 + (x2 / DST_L_B2) * (int)s[2]);)==""\n"
R"==(})==""\n"
R"==(int dst_i_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % DST_I_B0) * DST_I_SB0 + (x0 / DST_I_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DST_I_B1) * DST_I_SB1 + (x1 / DST_I_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DST_I_B2) * DST_I_SB2 + (x2 / DST_I_B2) * (int)s[2])==""\n"
R"==(+ (x3 % DST_I_B3) * DST_I_SB3 + (x3 / DST_I_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(#if WITH_DST_ITER_C)==""\n"
R"==(int dst_i_c_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % DST_I_C_B0) * DST_I_C_SB0 + (x0 / DST_I_C_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DST_I_C_B1) * DST_I_C_SB1 + (x1 / DST_I_C_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DST_I_C_B2) * DST_I_C_SB2 + (x2 / DST_I_C_B2) * (int)s[2])==""\n"
R"==(+ (x3 % DST_I_C_B3) * DST_I_C_SB3 + (x3 / DST_I_C_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(#if !IS_FWD)==""\n"
R"==(int diff_src_l_off(int64x3_t strides, int x0, int x1, int x2) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % DIFF_SRC_L_B0) * DIFF_SRC_L_SB0)==""\n"
R"==(+ (x0 / DIFF_SRC_L_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DIFF_SRC_L_B1) * DIFF_SRC_L_SB1)==""\n"
R"==(+ (x1 / DIFF_SRC_L_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DIFF_SRC_L_B2) * DIFF_SRC_L_SB2)==""\n"
R"==(+ (x2 / DIFF_SRC_L_B2) * (int)s[2]);)==""\n"
R"==(})==""\n"
R"==(int diff_dst_l_off(int64x3_t strides, int iter, int batch, int dhc) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((iter % DIFF_DST_L_B0) * DIFF_DST_L_SB0)==""\n"
R"==(+ (iter / DIFF_DST_L_B0) * (int)s[0])==""\n"
R"==(+ (batch % DIFF_DST_L_B1) * DIFF_DST_L_SB1)==""\n"
R"==(+ (batch / DIFF_DST_L_B1) * (int)s[1])==""\n"
R"==(+ (dhc % DIFF_DST_L_B2) * DIFF_DST_L_SB2)==""\n"
R"==(+ (dhc / DIFF_DST_L_B2) * (int)s[2]);)==""\n"
R"==(})==""\n"
R"==(int diff_src_i_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % DIFF_SRC_I_B0) * DIFF_SRC_I_SB0)==""\n"
R"==(+ (x0 / DIFF_SRC_I_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DIFF_SRC_I_B1) * DIFF_SRC_I_SB1)==""\n"
R"==(+ (x1 / DIFF_SRC_I_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DIFF_SRC_I_B2) * DIFF_SRC_I_SB2)==""\n"
R"==(+ (x2 / DIFF_SRC_I_B2) * (int)s[2])==""\n"
R"==(+ (x3 % DIFF_SRC_I_B3) * DIFF_SRC_I_SB3)==""\n"
R"==(+ (x3 / DIFF_SRC_I_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(int diff_dst_i_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % DIFF_DST_I_B0) * DIFF_DST_I_SB0)==""\n"
R"==(+ (x0 / DIFF_DST_I_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DIFF_DST_I_B1) * DIFF_DST_I_SB1)==""\n"
R"==(+ (x1 / DIFF_DST_I_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DIFF_DST_I_B2) * DIFF_DST_I_SB2)==""\n"
R"==(+ (x2 / DIFF_DST_I_B2) * (int)s[2])==""\n"
R"==(+ (x3 % DIFF_DST_I_B3) * DIFF_DST_I_SB3)==""\n"
R"==(+ (x3 / DIFF_DST_I_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(#if WITH_SRC_ITER_C)==""\n"
R"==(int diff_src_i_c_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % DIFF_SRC_I_C_B0) * DIFF_SRC_I_C_SB0)==""\n"
R"==(+ (x0 / DIFF_SRC_I_C_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DIFF_SRC_I_C_B1) * DIFF_SRC_I_C_SB1)==""\n"
R"==(+ (x1 / DIFF_SRC_I_C_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DIFF_SRC_I_C_B2) * DIFF_SRC_I_C_SB2)==""\n"
R"==(+ (x2 / DIFF_SRC_I_C_B2) * (int)s[2])==""\n"
R"==(+ (x3 % DIFF_SRC_I_C_B3) * DIFF_SRC_I_C_SB3)==""\n"
R"==(+ (x3 / DIFF_SRC_I_C_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(#if WITH_DST_ITER_C)==""\n"
R"==(int diff_dst_i_c_off(int64x4_t strides, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = strides.array;)==""\n"
R"==(return ((x0 % DIFF_DST_I_C_B0) * DIFF_DST_I_C_SB0)==""\n"
R"==(+ (x0 / DIFF_DST_I_C_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DIFF_DST_I_C_B1) * DIFF_DST_I_C_SB1)==""\n"
R"==(+ (x1 / DIFF_DST_I_C_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DIFF_DST_I_C_B2) * DIFF_DST_I_C_SB2)==""\n"
R"==(+ (x2 / DIFF_DST_I_C_B2) * (int)s[2])==""\n"
R"==(+ (x3 % DIFF_DST_I_C_B3) * DIFF_DST_I_C_SB3)==""\n"
R"==(+ (x3 / DIFF_DST_I_C_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(int diff_bias_off(int64x4_t diff_bias, int x0, int x1, int x2, int x3) {)==""\n"
R"==(int64_t *s = diff_bias.array;)==""\n"
R"==(return ((x0 % DIFF_BIAS_B0) * DIFF_BIAS_SB0)==""\n"
R"==(+ (x0 / DIFF_BIAS_B0) * (int)s[0])==""\n"
R"==(+ (x1 % DIFF_BIAS_B1) * DIFF_BIAS_SB1)==""\n"
R"==(+ (x1 / DIFF_BIAS_B1) * (int)s[1])==""\n"
R"==(+ (x2 % DIFF_BIAS_B2) * DIFF_BIAS_SB2)==""\n"
R"==(+ (x2 / DIFF_BIAS_B2) * (int)s[2])==""\n"
R"==(+ (x3 % DIFF_BIAS_B3) * DIFF_BIAS_SB3)==""\n"
R"==(+ (x3 / DIFF_BIAS_B3) * (int)s[3]);)==""\n"
R"==(})==""\n"
R"==(#endif )==""\n"
R"==(#endif)==""\n"
R"==()==";
}
}
}
}
}